# Spyt
# by Eresse <eresse@eresse.net>

# Internal Includes
require 'spyt/version'
require 'spyt/levels'
require 'spyt/config'
require 'spyt/loggable'

# Spyt Module:
# Root Module for Spyt.
module Spyt

	# Generic Log Handler:
	# Route log calls to each level.
	def self.method_missing name, *args

		# Fail on unknown Methods
		super unless LEVELS.include? name

		# Push Log (Level shortcuts)
		push name, args[0], args[1]
	end

	# Push Message:
	# Writes a message _msg_ from _src_ as level _lvl_ to the log.
	# @param [Symbol] lvl Log level (see LEVELS)
	# @param [String] src Message source
	# @param [String] msg Message text
	def self.push lvl, src, msg

		# Check Level
		raise "Invalid Log Level [#{lvl.to_s.inspect.bred}] requested" unless LEVELS.include? lvl.to_sym

		# Check Level
		return unless LEVELS[lvl] <= LEVELS[CONFIG[:lvl]]

		# Ensure Lock is available
		@lock ||= Mutex.new

		# Synchronize
		@lock.synchronize do

			# Write & Flush
			CONFIG[:out] << "#{Time.now.to_s.blue} [#{lvl.to_s.send LEVEL_COLS[lvl.to_sym]}] #{src.bmagenta} :: #{msg}\n"
			CONFIG[:out].flush if CONFIG[:out].respond_to? :flush
		end
	end
end
